package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;
import java.util.Vector;

// Spelchan Imports
import com.blazinggames.urcl.data.Config;
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.sprite.*;
import com.blazinggames.urcl.ui.playfield.*;
import com.blazinggames.urcl.ui.comp.*;

// Retro Imports
import com.blazinggames.retro.shared.search.*;

/**
 * Ultimate Retro Project Episode 10: Modern Tank Battle
 * Copyright (C) 2001, 2004, 2006 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.10.060203

*/
public class TankDisplay 
extends Display
{
// *********************
// ***** CONSTANTS *****
// *********************

    private static final Point DEF_PLACE = new Point (0,0);
    private static final Rectangle DEF_RECT = new Rectangle(0,0,0,0);

    // Shared constants (also used by TankTitle)
    private static final String PLAYERCOLORS = "10PLAYERCOLOR";
    private static final String PLAYERNAMES = "10PLAYERNAME";

    // Tank Specific configuration - images
    private static final String BACKDROP_IMAGE = "10BACKDROP";
    private static final String TANKICONS = "10TANKICONS";
    private static final String TANKWHEEL = "10WHEEL";
    private static final String TANKSPEED = "10SPEED";

    // Tank Specific configuration - component layout
    private static final String MESSAGESPOT = "10MESSAGESPOT";
    private static final String PLAYFIELD = "10PLAYFIELD";
    private static final String WHEELSPOT = "10WHEELSPOT";
    private static final String SPEEDSPOT = "10SPEEDSPOT";
    private static final String TURRETDIRECTIONSPOT = "10TURRETDIRSPOT";
    private static final String TURRETANGLESPOT = "10TURRETANGSPOT";
    private static final String FIRINGPOWERSPOT = "10POWERSPOT";
    private static final String DONEBUTTONSPOT = "10DONESPOT";
    private static final String DIRECTIONLABEL = "10DIRLABEL";
    
    // Sound support
    private static final String USE_SOUND = "10USESOUND";
    private static final String SOUND_IMAGE = "10SOUNDIMAGE";
    private static final String SOUND_RECT = "10SOUNDRECT";
    private static final String SHOOT_SOUND = "10SHOOTSOUND";
    private static final String HIT_SOUND = "10HITSOUND";
    private static final String MISS_SOUND = "10MISSSOUND";
    private static final String WIN_SOUND = "10WINSOUND";

    // BUTTON CONSTANTS
    private static final int BTN_BIGDEC_POWER = 0;
    private static final int BTN_DECREMENT_POWER = 1;
    private static final int BTN_INCREMENT_POWER = 2;
    private static final int BTN_BIGINC_POWER = 3;
    private static final int BTN_DONE = 4;
    private static final int TOTAL_BUTTONS = 5;
    
// *********************
// ***** VARIABLES *****
// *********************

    Image _background;
    java.util.Random _random;
    boolean _fullPaint;
    boolean _gameOver;
    int _nextTurnDelay;

    int _explosionRadius;
    
    // layout
    private Rectangle _messageSpot;
    private Rectangle _playfield;
    private Rectangle _wheelSpot;
    private Rectangle _speedSpot;
    private Rectangle _turretDirSpot;
    private Rectangle _turretAngSpot;
    private Rectangle _powerSpot;
    private Rectangle _doneSpot;
    private Rectangle _shotBounds;
    
    // Components 
    ImageStripSprite _wheelSprite;
    ImageStripSprite _speedSprite;
    TextButton _buttons[];
    SLabel _powerLabel;
    SLabel _message;
    SLabel _facingLabel;
    SLabel _turretDirLabel;
    SLabel _turretAngLabel;
    
    // Player information
    private boolean _playerAlive[];
    private int _playerType[];
    private int _playerTurn[];
    private int _playerSpeed[];
    private double _playerTDir[];
    private int _playerTAng[];
    private int _playerPower[];
    private Tank _playerTank[];
    private ImageStripSprite _playerSprite[];
    private Sprite _lastShotSprite[];
    private Color _playerColors[][];
    private String _playerNames[];
    private int _curPlayer;
    private Projectile _curProjectile;
    
    // Sound management
    boolean _useSound;
    Toggle _soundToggle;
    private Rectangle _soundRect;
    
    // AI
    private int _aiLastTarget[];
    //    private int _aiLastPower[];
    //    private boolean _aiWasLastShotLong[];
    private int _aiLowPower[];
    private int _aiHighPower[];
    

// *****************************************
// ***** METHODS - Initialization aids *****
// *****************************************

    /**
     * gets the resource manager to preload all the sprite images used in the game
    */
    private void prepareSprites()
    {
        Container c = _displayHandler.getContainer();
        
        _res.addImage(TANKICONS, _config.getStringProperty(TANKICONS, "tanks.gif"));
        _res.addImage(TANKWHEEL, _config.getStringProperty(TANKWHEEL, "wheel.gif"));
        _res.addImage(TANKSPEED, _config.getStringProperty(TANKSPEED, "speed.gif"));

        _playerSprite = new ImageStripSprite[8];
        _lastShotSprite = new Sprite[8];
        for (int cntr = 0; cntr < 8; ++cntr)
        {
            _playerTank[cntr] = new Tank();
            _playerSprite[cntr] = new ImageStripSprite(c, _res.getImage(TANKICONS), 8, 8);
            _playerSprite[cntr].setVisible(false);
            _lastShotSprite[cntr] = new Sprite();
            _lastShotSprite[cntr].setSolid(true);
            _lastShotSprite[cntr].setBackground(_playerColors[cntr][1]);
        }
    }

    private void prepareSounds()
    {
        _useSound = (_config.getIntProperty(USE_SOUND, 0) == 1);
        _res.addImage(SOUND_IMAGE, _config.getStringProperty(SOUND_IMAGE, "sound.gif"));
        Rectangle r = _config.getRectangleProperty(SOUND_RECT, DEF_RECT);
        _soundRect = r;
    	_soundToggle = new Toggle(_displayHandler.getContainer(),
                    _res.getImage(SOUND_IMAGE), r.x, r.y, r.width, r.height, 2);
        if (_useSound)
        {
            _res.addAudioClip(SHOOT_SOUND, _config.getStringProperty(SHOOT_SOUND, "silent.au"));
            _res.addAudioClip(HIT_SOUND, _config.getStringProperty(HIT_SOUND, "silent.au"));
            _res.addAudioClip(MISS_SOUND, _config.getStringProperty(MISS_SOUND, "silent.au"));
            _res.addAudioClip(WIN_SOUND, _config.getStringProperty(WIN_SOUND, "silent.au"));
        }
    }

    /**
     * Initializes all the player data
    */
    private void preparePlayers()
    {
        _playerAlive = new boolean[8];
        _playerType = new int[8];
    	_playerTurn = new int[8];
    	_playerSpeed = new int[8];
    	_playerTDir = new double[8];
    	_playerTAng = new int[8];
    	_playerPower = new int[8];
        _playerTank = new Tank[8];
    	_curPlayer = 0;

        _playerColors = new Color[8][3];
        _playerNames = new String[8];
        for (int cntr = 0; cntr < 8; ++cntr)
        {
            _playerNames[cntr] = _config.getStringProperty(PLAYERNAMES + "[" + cntr + "]", "Player " + (cntr+1));
            _playerColors[cntr][1] = _config.getColorProperty(PLAYERCOLORS + "[" + cntr + "]", Color.white);
            _playerColors[cntr][2] = _playerColors[cntr][1].brighter();
            _playerColors[cntr][0] = _playerColors[cntr][1].darker();
        }

        _aiLastTarget = new int[8];
        //    private int _aiLastPower[];
        //    private boolean _aiWasLastShotLong[];
        _aiLowPower = new int[8];
        _aiHighPower = new int[8];
    }
    
    /**
     * processes the layout
    */
    public void prepareLayout()
    {
        Container c = _displayHandler.getContainer();
        _playerType = new int[8];

    	_messageSpot = _config.getRectangleProperty(MESSAGESPOT, DEF_RECT);
        _message = new SLabel(c, "?", _messageSpot);
    	_playfield = _config.getRectangleProperty(PLAYFIELD, DEF_RECT);
        _shotBounds = new Rectangle(0,0, _playfield.width * 10, _playfield.height * 10);
        
        _wheelSpot = _config.getRectangleProperty(WHEELSPOT, DEF_RECT);
        _wheelSprite = new ImageStripSprite(c, _res.getImage(TANKWHEEL), _wheelSpot.width, _wheelSpot.height);
        _wheelSprite.move(_wheelSpot.x, _wheelSpot.y);
        _wheelSprite.setVisible(true);

    	_speedSpot = _config.getRectangleProperty(SPEEDSPOT, DEF_RECT);
        _speedSprite = new ImageStripSprite(c, _res.getImage(TANKSPEED), _speedSpot.width, _speedSpot.height);
        _speedSprite.move(_speedSpot.x, _speedSpot.y);
        _speedSprite.setVisible(true);
        
    	_turretDirSpot = _config.getRectangleProperty(TURRETDIRECTIONSPOT, DEF_RECT);
    	_turretAngSpot = _config.getRectangleProperty(TURRETANGLESPOT, DEF_RECT);
     	_turretDirLabel = new SLabel(c, "dir", _turretDirSpot.x, _turretDirSpot.y + _turretDirSpot.height - 20, _turretDirSpot.width, 20);
    	_turretAngLabel = new SLabel(c, "dir", _turretAngSpot.x, _turretAngSpot.y + _turretAngSpot.height, _turretAngSpot.width, 20);

        _powerSpot = _config.getRectangleProperty(FIRINGPOWERSPOT, DEF_RECT);
    	_doneSpot = _config.getRectangleProperty(DONEBUTTONSPOT, DEF_RECT);
        int powWidth =  _powerSpot.width / 6;
        _buttons = new TextButton[TOTAL_BUTTONS];
        _buttons[BTN_BIGDEC_POWER] = new TextButton(c, "<<", _powerSpot.x, _powerSpot.y, powWidth, _powerSpot.height);
        _buttons[BTN_DECREMENT_POWER] = new TextButton(c, "<", _powerSpot.x+powWidth, _powerSpot.y, powWidth, _powerSpot.height);
        _powerLabel = new SLabel(c, "?", _powerSpot.x+powWidth*2, _powerSpot.y, powWidth*2,_powerSpot.height);
        _buttons[BTN_INCREMENT_POWER] = new TextButton(c, ">", _powerSpot.x+powWidth*4, _powerSpot.y, powWidth, _powerSpot.height);
    	_buttons[BTN_BIGINC_POWER] = new TextButton(c, ">>", _powerSpot.x+powWidth*5, _powerSpot.y, powWidth, _powerSpot.height);
        _buttons[BTN_DONE] = new TextButton(c, "DONE", _doneSpot.x, _doneSpot.y, _doneSpot.width, _doneSpot.height);

        _facingLabel = new SLabel(c, "Direction: ?", _config.getRectangleProperty(DIRECTIONLABEL, DEF_RECT));
    }


// ***********************************
// ***** METHODS -  From Display *****
// ***********************************

    /**
     * Initializes the game (using private helper methods) by getting necessary
     * resources and setting up configurable variables.
     *
     * @param dh the controlling applet/application
    */
    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);
        _random = new java.util.Random();
        dh.displayEvent(this, "LOADING 20");
        _res.addImage(BACKDROP_IMAGE, _config.getStringProperty(BACKDROP_IMAGE, "backdrop.jpg"));
        _background = _res.getImage(BACKDROP_IMAGE);
        dh.displayEvent(this, "LOADING 40");
        prepareSounds();
        preparePlayers();
        dh.displayEvent(this, "LOADING 60");
        prepareSprites();
        dh.displayEvent(this, "LOADING 80");
        prepareLayout();
        dh.displayEvent(this, "LOADING 100");

        return true;
    }


    public boolean begin()
    {
        _fullPaint = true;
        return super.begin();
    }


    /**
     * Handles mouse movement. We may need to update the current guess variable
     * and if the game is over, the quit button needs messages.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
     * @param b true if mouse button down
    */
    public void mouse_move(int x, int y, boolean b)
    {
        boolean repaint = false;
        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_move(x, y, b);
        }
        if (repaint)
            _displayHandler.refreshDisplay(this);
    }
    
    /**
     * Handles the mouse being clicked. If over the hallway (playfield) then
     * player may be starting to make a guess. If the game is over then we
     * have to pass messages to the quit button.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
    */
    public void mouse_down(int x, int y)
    {
        boolean repaint = false;
        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_down(x, y);
        }
        repaint |= _soundToggle.mouse_down(x, y);
        
        if (repaint)
            _displayHandler.refreshDisplay(this);
    }
    
    /**
     * Handles the mouse button being released. This sends messages to the
     * scrollbar and (if game over) quit button components as well as handling
     * clicking on a door in the hallway playfield.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
    */
    public void mouse_up(int x, int y)
    {
        boolean repaint = true;
        int temp;
        
        if (_gameOver)
        {
            _displayHandler.displayEvent(this, "TANK_TITLE");
            return;
        }
        
        if (_curProjectile != null)
            ; // do nothing
        else if (_wheelSpot.contains(x, y)) 
        {
            temp = x - _wheelSpot.x - 64;
            if (temp < 0)
                _wheelSprite.setFrame(Math.max(0, _wheelSprite.getFrame() - 1));
            else
                _wheelSprite.setFrame(Math.min(4, _wheelSprite.getFrame() + 1));
        }
        else if (_speedSpot.contains(x, y))
        {
            _speedSprite.setFrame( 3 - ((y - _speedSpot.y) / 32));
        }
        else if (_turretDirSpot.contains(x, y))
        {
            _playerTDir[_curPlayer] = findAngle(_turretDirSpot.x+64, _turretDirSpot.y+64, x, y);
            updateAngles();
        }
        else if (_turretAngSpot.contains(x, y))
        {
            _playerTAng[_curPlayer] = (int)findAngle(_turretAngSpot.x, _turretAngSpot.y+64, x, y);
            updateAngles();
        }

        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            if (_buttons[cntr].mouse_up(x, y))
            {
                repaint = true;
                handleButton(cntr);
            }
        }
        repaint |= _soundToggle.mouse_up(x, y);
        if (repaint)
            _displayHandler.refreshDisplay(this);
    }
    
    /**
     * Ticks occur at a specified interval (100ms in my case).
     *
     * @param ms milliseconds between this click and the last one
    */
    public void tick(int ms)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            repaint |= _buttons[cntr].tick(ms);
        }
        repaint |= _soundToggle.tick(ms);
        
        if (_curProjectile != null)
        {
            moveProjectile();
            repaint = true;
        }

        if (_nextTurnDelay > 0)
        {
            --_nextTurnDelay;
            if (_nextTurnDelay <= 0)
            {
                nextPlayer();
            }
        }
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

    private void moveProjectile()
    {
        int cntr;
        
        _curProjectile.tick(200);
        if (_curProjectile.isAlive() == false)
        {
            Rectangle blast = new Rectangle(
                    (int)(_curProjectile.getX() - _explosionRadius),
                    (int)(_curProjectile.getY() - _explosionRadius),
                    _explosionRadius * 2, _explosionRadius * 2);
            _lastShotSprite[_curPlayer].setBounds(blast.x / 10 + _playfield.x, blast.y / 10 + _playfield.y, blast.width / 10, blast.height / 10);
            _lastShotSprite[_curPlayer].setVisible(true);
            boolean hitTank = false;
            for (cntr = 0; cntr < 8; ++cntr)
            {
                if (blast.contains((int)_playerTank[cntr].getX(), (int)_playerTank[cntr].getY()))
                {
                    hitTank = true;
                    _playerAlive[cntr] = false;
                    _playerSprite[cntr].setVisible(false);
                    _lastShotSprite[cntr].setVisible(false);
                    _displayHandler.log("Tank " + cntr + " destroyed.");
                    _message.setText(_displayHandler.getContainer(),
                                     _playerNames[_curPlayer] + " destroyed " + _playerNames[cntr]);
                }
            }
            aiAnalyseShot(_curPlayer, _curProjectile.getX(), _curProjectile.getY());
            _curProjectile = null;
            if (hitTank)
                playSound(HIT_SOUND);
            else
                playSound(MISS_SOUND);
            _nextTurnDelay = 15;
//            nextPlayer();
        }
        else if (_shotBounds.contains((int)_curProjectile.getX(), (int)_curProjectile.getY()) == false)
        {
            // for ai, we need to know where the hit zone is
            while (_curProjectile.isAlive() == false)
                _curProjectile.tick(200);
            aiAnalyseShot(_curPlayer, _curProjectile.getX(), _curProjectile.getY());
            _curProjectile = null;
            _lastShotSprite[_curPlayer].setVisible(false);
            _message.setText(_displayHandler.getContainer(), _playerNames[_curPlayer] + "\'s shot out of bounds!");
            _nextTurnDelay = 15;
//            nextPlayer();
        }
    }


    /**
     * draws the display screen to the indicated graphics context.
     *
     * @param g Graphics context to draw to
     * @param comp Component for awt calls
    */
    public synchronized void render(Graphics g, Component comp)
    {
        int cntr;
        
        if (_background == null)
        {
            g.setColor(Color.blue);
            g.fillRect(0,0,640,480);
        }
        else
        {
            g.drawImage(_background, 0, 0, comp);
        }
/*
        g.setColor(Color.pink);
    	drawRect(g, _messageSpot);
    	drawRect(g, _playfield);
    	drawRect(g, _wheelSpot);
    	drawRect(g, _speedSpot);
    	drawRect(g, _turretDirSpot);
    	drawRect(g, _turretAngSpot);
    	drawRect(g, _powerSpot);
    	drawRect(g, _doneSpot);
*/        
        _wheelSprite.render(g, comp);
        _speedSprite.render(g, comp);

        for (cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            _buttons[cntr].render(g, comp);
        }
        _powerLabel.render(g, comp);
        
        for (cntr = 0; cntr < 8; ++cntr)
        {
            _playerSprite[cntr].render(g, comp);
        }
        for (cntr = 0; cntr < 8; ++cntr)
        {
            _lastShotSprite[cntr].render(g, comp);
        }
        
        drawProjectile(g);
        
        g.setColor(_playerColors[_curPlayer][1]);
        drawAngle(g, _turretDirSpot.x + 64, _turretDirSpot.y + 64, _playerTDir[_curPlayer], 64, 5);
        drawAngle(g, _turretAngSpot.x, _turretAngSpot.y + 64, (double)_playerTAng[_curPlayer], 64, 5);
        
        _message.render(g, comp);
        _facingLabel.render(g, comp);
    	_turretDirLabel.render(g, comp);
    	_turretAngLabel.render(g, comp);

        _soundToggle.render(g, comp);
        if (_soundToggle.getState() == 1)
        {
            g.setColor(Color.red);
            g.drawRect(_soundRect.x, _soundRect.y, _soundRect.width, _soundRect.height);
        }
    }

/*
    private void drawRect(Graphics g, Rectangle r)
    {
        g.drawRect(r.x, r.y, r.width, r.height);
    }
*/
    
    private void drawAngle(Graphics g, int x, int y, double ang, int size, int thick)
    {
        int xpoints[] = new int[5];
        int ypoints[] = new int[5];
        double rad, xoff, yoff, boxAng;
        
        boxAng = ang + 90.0;
        if (boxAng >= 360.0)
            boxAng -= 360.0;
        rad = boxAng * Math.PI / 180.0;
        xoff = Math.cos(rad) * thick;
        yoff = Math.sin(rad) * thick;
        xpoints[0] = x + (int)xoff;
        ypoints[0] = y - (int)yoff;
        xpoints[1] = x - (int)xoff;
        ypoints[1] = y + (int)yoff;

/*
        boxAng = ang - 90.0;
        if (boxAng < 0.0)
            boxAng = 360.0 + boxAng;
        rad = boxAng * Math.PI / 180.0;
        xoff = Math.cos(rad) * thick;
        yoff = Math.sin(rad) * thick;
        xpoints[1] = x + (int)xoff;
        ypoints[1] = y - (int)yoff;
*/
        rad = ang * Math.PI / 180.0;
        xoff = Math.cos(rad) * size;
        yoff = Math.sin(rad) * size;
        xpoints[2] = xpoints[1] + (int)xoff;
        ypoints[2] = ypoints[1] - (int)yoff;
        xpoints[3] = xpoints[0] + (int)xoff;
        ypoints[3] = ypoints[0] - (int)yoff;
        xpoints[4] = xpoints[0];
        ypoints[4] = ypoints[0];

        g.fillPolygon(xpoints, ypoints, 5);
    }

    private void drawProjectile(Graphics g)
    {
        try
        {
            int x = (int)(_curProjectile.getX() / 10);
            int y = (int)(_curProjectile.getY() / 10);
            int h = Math.max(3, (int)(_curProjectile.getHeight() / 250) );
            int hh = h / 2;
            x += _playfield.x;
            x -= hh;
            y += _playfield.y;
            y -= hh;
            g.setColor(Color.white);
            g.drawOval(x-2, y-2, h+4, h+4);
            g.fillOval(x, y, h, h);
        }
        catch (NullPointerException npe)
        {
            ; // just ignore as object doesnt exist or landed while frame drawn
        }
    }
    
    /**
     * If sound is enabled gets the resource manager to play the indicated sound.
     *
     * @param key sound to play
    */
    private void playSound(String key)
    {
        if ( (_useSound) && (_soundToggle.getState() == 1) )
        {
            _res.playAudio(key);
        }
    }


// ***********************************************
// ***** METHODS - User Interface Management *****
// ***********************************************

    public double findAngle(int x1, int y1, int x2, int y2)
    {
        int distX = x2 - x1;
        int distY = y2 - y1;
        int absX = Math.abs(distX);
        int absY = Math.abs(distY);
        if (absX == 0)
        {
            if (distY > 0)
                return 270.0;
            else
                return 90.0;
        }
        double tan = (double)absY / (double)absX;
        double atan = Math.atan(tan);
        double deg = atan / Math.PI * 180.0;
        if (distX < 0)
        {
            if (distY < 0)
            {
                return (180.0 - deg);
            }
            else
            {
                return (180.0 + deg);
            }
        }
        else
        {
            if (distY < 0)
            {
                return (deg);
            }
            else
            {
                return (360.0 - deg);
            }
        }
    }

    private void setButtonState(int player, boolean visible)
    {
        Container c = _displayHandler.getContainer();

        for (int cntr = 0; cntr < TOTAL_BUTTONS; ++cntr)
        {
            _buttons[cntr].setNormalColors(_playerColors[player][0], Color.black);
            _buttons[cntr].setOverColors(_playerColors[player][1], Color.black);
            _buttons[cntr].setSelectedColors(_playerColors[player][2], Color.black);
            _buttons[cntr].setVisible(visible);
        }
        _powerLabel.setText(c, ""+_playerPower[_curPlayer]);
        if (visible)
        {
            _wheelSprite.setFrame(2);
            _speedSprite.setFrame(1);
        }
        updateAngles();
    }

    private void updateAngles()
    {
        Container c = _displayHandler.getContainer();

        _facingLabel.setText(c, "Facing " + (360 - _playerTank[_curPlayer].getDirection()) + " deg");
        int n, m, s;
        double dtemp;
        n = (int)_playerTDir[_curPlayer];
        dtemp = _playerTDir[_curPlayer] - (double)n;
        m = (int)(dtemp * 60);
        s = (int)((dtemp * 3600) % 60);
    	_turretDirLabel.setText(c, ""+ n + "deg " + m +"min " + s + "sec");
    	_turretAngLabel.setText(c, ""+_playerTAng[_curPlayer] + "deg");
    }
    
    private void handleButton(int id)
    {
        switch (id)
        {
            case BTN_BIGDEC_POWER:
                _playerPower[_curPlayer] = Math.max(1, _playerPower[_curPlayer] - 10);
                break;
                
            case BTN_DECREMENT_POWER:
                _playerPower[_curPlayer] = Math.max(1, _playerPower[_curPlayer] - 1);
                break;
                
            case BTN_INCREMENT_POWER:
                _playerPower[_curPlayer] = Math.min(1000, _playerPower[_curPlayer] + 1);
                break;
                
            case BTN_BIGINC_POWER:
                _playerPower[_curPlayer] = Math.min(1000, _playerPower[_curPlayer] + 10);
                break;
                
            case BTN_DONE:
                doMove();
                break;
        }
        _powerLabel.setText(_displayHandler.getContainer(), ""+_playerPower[_curPlayer]);
    }

// ***********************************
// ***** METHODS - Game Handling *****
// ***********************************

    public int rand(int low, int high)
    {
        int r = high - low + 1;
        if (r == 1)
            return low;
        return (((_random.nextInt() & Integer.MAX_VALUE) % r) + low);
    }

    public void setPlayerType(int id, int type)
    {
        _playerType[id] = type;
    }

    public void setExplosionRadius(int radius)
    {
        _explosionRadius = 10 * (radius * 2 + 1) + 5;

        _displayHandler.log("Explosion Radius set for " + _explosionRadius); 
    }
    
    public void startGame()
    {
        boolean valid;
        int cntr, cntrT;
        double placeX, placeY, checkX, checkY;
        
        for (cntr = 0; cntr < 8; ++cntr)
        {
            do
            {
                valid = true;
                placeX = rand(20,379) * 10;
                placeY = rand(20,379) * 10;

                for (cntrT = 0; cntrT < cntr; ++ cntrT)
                {
                    checkX = _playerTank[cntrT].getX();
                    checkY = _playerTank[cntrT].getY();
                    if ( ( (placeX > (checkX-100)) && (placeX < (checkX+100))) || ( (placeY > (checkY-100)) && (placeY < (checkY+100))) )
                        valid = false;
                } 
            }
            while ( ! valid);
            _playerTank[cntr].setLocation(placeX, placeY);
            _playerTank[cntr].setDirection(0);
            _playerSprite[cntr].move((int)(_playerTank[cntr].getX() / 10 + _playfield.x-4),(int)(_playerTank[cntr].getY() / 10 + _playfield.y-4));
            _playerSprite[cntr].setFrame(cntr * 8);
            _playerAlive[cntr] = _playerType[cntr] > 0;
            _playerSprite[cntr].setVisible(_playerAlive[cntr]);
            _lastShotSprite[cntr].setVisible(false);
            _aiLastTarget[cntr] = -1;
//System.out.println("X=" + _playerSprite[cntr].getX() + "," + _playerSprite[cntr].getY());
            _playerTurn[cntr] = 0;
            _playerSpeed[cntr] = 0;
            _playerTDir[cntr] = 0.0;
            _playerTAng[cntr] = 45;
            _playerPower[cntr] = 100;
        }
        _gameOver = false;
        _curPlayer = 7;
        _nextTurnDelay = 1;
//        nextPlayer();
    }

    private void doMove()
    {
        _playerTank[_curPlayer].turn(45 * _wheelSprite.getFrame() - 90);
        int dir = _playerTank[_curPlayer].getDirection();
        int move = _speedSprite.getFrame() - 1;
        handleTankMovement(dir, move);
        _playerTank[_curPlayer].setFireDirection(360-_playerTDir[_curPlayer]);
        _playerTank[_curPlayer].setFireAngle(_playerTAng[_curPlayer]);
        _playerTank[_curPlayer].setFirePower(_playerPower[_curPlayer]);
        _curProjectile = _playerTank[_curPlayer].fire();
        playSound(SHOOT_SOUND);
        setButtonState(_curPlayer, false);
    }

    private void handleTankMovement(int facing, int speed)
    {
        int dir = facing / 45;
        int move = 10 * speed;
        switch (dir)
        {
            case 0: // east
                _playerTank[_curPlayer].setLocation(Math.min(_playerTank[_curPlayer].getX()+move, _shotBounds.width), _playerTank[_curPlayer].getY());
                break;

            case 1: // south east
                _playerTank[_curPlayer].setLocation(Math.min(_playerTank[_curPlayer].getX()+move, _shotBounds.width), Math.min(_shotBounds.height, _playerTank[_curPlayer].getY()+move));
                break;

            case 2: // south
                _playerTank[_curPlayer].setLocation(_playerTank[_curPlayer].getX(), Math.min(_playerTank[_curPlayer].getY()+move,_shotBounds.height));
                break;

            case 3: // south west
                _playerTank[_curPlayer].setLocation(Math.max(0, _playerTank[_curPlayer].getX()-move), Math.min(_shotBounds.height, _playerTank[_curPlayer].getY()+move));
                break;

            case 4: // west
                _playerTank[_curPlayer].setLocation(Math.max(0, _playerTank[_curPlayer].getX()-move), _playerTank[_curPlayer].getY());
                break;

            case 5: // north west
                _playerTank[_curPlayer].setLocation(Math.max(0, _playerTank[_curPlayer].getX()-move), Math.max(0, _playerTank[_curPlayer].getY()-move));
                break;

            case 6: // north in our system
                _playerTank[_curPlayer].setLocation(_playerTank[_curPlayer].getX(), Math.max(_playerTank[_curPlayer].getY()-move, 0));
                break;

            case 7: // north east
                _playerTank[_curPlayer].setLocation(Math.min(_shotBounds.width, _playerTank[_curPlayer].getX()+move), Math.max(0, _playerTank[_curPlayer].getY()-move));
                break;
        }
        _playerSprite[_curPlayer].move((int)(_playerTank[_curPlayer].getX() / 10 + _playfield.x-4),(int)(_playerTank[_curPlayer].getY() / 10 + _playfield.y-4));
        _playerSprite[_curPlayer].setFrame(_curPlayer * 8 + dir);        
    }
    
    /**
     * Checks to see if the game is over. If it isn't finds out who the next
     * player is and sets the game up to use that player.
     */
    private void nextPlayer()
    {
        Container c = _displayHandler.getContainer();
        int tankCount = 0;
        int humanCount = 0;
        int lastCounted = 0;
        for (int cntr = 0; cntr < 8; ++cntr)
        {
            if (_playerAlive[cntr])
            {
                ++tankCount;
                lastCounted = cntr;
                if (_playerType[cntr] == 1)
                    ++humanCount;
            }
        }
        if (tankCount == 1)
        {
            playSound(WIN_SOUND);
            _message.setText(c, _playerNames[lastCounted] + " won! Click to return to title");
            _gameOver = true;
            _curPlayer = lastCounted;
            setButtonState(lastCounted, false);
            return;
        }
        else if (tankCount == 0)
        {
            _message.setText(c, "NO WINNER! Click to return to title");
            _gameOver = true;
            setButtonState(_curPlayer, false);
            return;
        }
        else if (humanCount == 0)
        {
            _message.setText(c, "ONLY COMPUTER LEFT! Click to return to title");
            _gameOver = true;
            setButtonState(_curPlayer, false);
            return;
        }
        
        // find next player
        boolean valid = false;
        do
        {
            ++_curPlayer;
            if (_curPlayer > 7)
            _curPlayer = 0;
            valid = _playerAlive[_curPlayer];
        }
        while (valid == false);

        _message.setText(c, _playerNames[_curPlayer] + "\'s turn");
        if (_playerType[_curPlayer] == 1)
            setButtonState(_curPlayer, true);
        else
            computerTurn();
    }

    private void computerTurn()
    {
        aiMovement();

        aiTargeting(_curPlayer);
//        nextPlayer();
    }


// ********************************
// ***** METHODS: AI Handling *****
// ********************************

    private double findAngle(int id, int target)
    {
        // s = source, t = target, d = delta
        double sx = _playerTank[id].getX();
        double tx = _playerTank[target].getX();
        double dx = Math.abs(sx - tx);
        double sy = _playerTank[id].getY();
        double ty = _playerTank[target].getY();
        double dy = Math.abs(sy - ty);
        double tanAng = 0;

        if (dx > 0)
            tanAng = dy / dx;
        else
            tanAng = dy / 0.000000001;
        double angRad = Math.atan(tanAng);
        double angDeg = angRad * 180 / Math.PI;

        if (sx < tx)
        {
            if (sy < ty) // quad I
                return angDeg;
            else // quad IV
                return 360 - angDeg;
        }
        else
        {
            if (sy < ty) // quad II
                return 180 - angDeg;
            else // quad III
                return 180 + angDeg;
        }
    }

     public void aiMovement()
     {
         int turnAngle = rand(-2, 2);
         int advAmount = rand(-1, 4);

        _playerTank[_curPlayer].turn(turnAngle*45);
        _wheelSprite.setFrame(turnAngle);
        _speedSprite.setFrame(advAmount);

         handleTankMovement(_playerTank[_curPlayer].getDirection(), advAmount);
     }

    public void aiTargeting(int id)
    {
        // first see if need to aquire new target
        if ( (_aiLastTarget[id] == -1) || (_playerType[id] == 2) )
            aiAquireTarget(id);
        else if (_playerAlive[_aiLastTarget[id]] == false)
            aiAquireTarget(id);

        // Next calculate targetting angle (computer cheats by getting new angle every turn)
        double angle = findAngle(id, _aiLastTarget[id]);
        int power, powerAdj;

        // Third, calculate new power to use
        _aiLowPower[id] = Math.max(_aiLowPower[id] - 5, 40);
        _aiHighPower[id] = Math.min(_aiHighPower[id] + 5, 1000);
        power = rand(_aiLowPower[id], _aiHighPower[id]);

        // Finally take the shot and update ai info
        _playerTank[id].setFireDirection(angle);
        _playerTank[id].setFireAngle(45);
        _playerTank[id].setFirePower(power);
        _playerTDir[id] = 360 - angle;
        _playerTAng[id] = 45;
        _playerPower[id] = power;

        setButtonState(id, false);
        _curProjectile = _playerTank[_curPlayer].fire();
        playSound(SHOOT_SOUND);
    }

    public void aiAquireTarget(int id)
    {
        // build potential valid list
        int potentialList[] = new int[8];
        int cntr, valid = 0, pick;

        for (cntr = 0; cntr < 8; ++cntr)
        {
            if ( (id != cntr) && (_playerAlive[cntr]))
            {
                potentialList[valid] = cntr;
                ++valid;
            }
        }

        // pick from list (or 0 if empty list)
        if (valid > 0)
        {
            pick = potentialList[rand(1, valid) - 1];
        }
        else
            pick = 0;
        _aiLastTarget[id] = pick;
        //        _aiWasLastShotLong[id] = false;
        //        _aiLastPower[id] = 50;
        _aiHighPower[id] = 350;
        _aiLowPower[id] = 50;
        _displayHandler.log("AI for " + id + " Choosing " + pick + " as new target");
    }

    private void aiAnalyseShot(int id, double hitX, double hitY)
    {
        if (_playerType[_curPlayer] < 3)
            return;
        double tempX = _playerTank[id].getX() - _playerTank[_aiLastTarget[id]].getX();
        double tempY = _playerTank[id].getY() - _playerTank[_aiLastTarget[id]].getY();
        double targDist = Math.sqrt(tempX * tempX + tempY*tempY);

        tempX = hitX - _playerTank[id].getX();
        tempY = hitY - _playerTank[id].getY();
        double hitDist = Math.sqrt(tempX * tempX + tempY * tempY);
        //        _aiLastPower[id] = power;
        _displayHandler.log("AI: Target distance = " + targDist + " actual = " + hitDist); 
        if (targDist < hitDist)
            _aiHighPower[id] = _playerTank[id].getFirePower();
        else
            _aiLowPower[id] = _playerTank[id].getFirePower();
    }
}
