package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.comp.*;
import com.blazinggames.urcl.ui.sprite.*;

// Retro imports
import com.blazinggames.retro.modern.Title;

/**
 * Ultimate Retro Project Episode 10: Modern Tank Battle
 * Copyright (C) 2001, 2004, 2006 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.10.060203
*/
public class TankTitle 
extends Title
{
// *********************
// ***** CONSTANTS *****
// *********************

    private static final int MAXPLAYERS = 8;

    // Tank title specific 
    private static final String PLAYERROWS = "10PLAYERROWS";
    private static final String PLAYERCOLS = "10PLAYERCOLS";
    private static final String EXPLOSIONROW = "10EXPLOSIONROW";
    private static final String EXPLOSIONCOLS = "10EXPLOSIONCOLS";
    private static final String EXPLOSIONLABELS = "10EXPLOSIONLABEL";
    
    // Shared constants (also used by TankDisplay)
    private static final String PLAYERCOLORS = "10PLAYERCOLOR";
    private static final String PLAYERNAMES = "10PLAYERNAME";
    

// *********************
// ***** VARIABLES *****
// *********************

    protected int _playerRows[];
    protected int _playerCols[];
    protected SRadioButton _radioButtons[];
    protected SRadioContainer _playerContainers[];
    protected SLabel _playerLabels[];
    protected SRadioButton _explosionButtons[];
    protected SRadioContainer _explosionContainer;

// ***********************
// ***** CONSTRUCTOR *****
// ***********************    

    public TankTitle()
    {
        super();
    }


    public TankTitle(String pref)
    {
        super(pref);
    }


// *****************************************
// ***** METHODS - Initialization aids *****
// *****************************************

    public void prepareRadioButtons()
    {
        Container c = _displayHandler.getContainer();
        Color pcolor;
        int n = MAXPLAYERS;
        int temp;
    
        _radioButtons = new SRadioButton[n * 4];
        _playerContainers = new SRadioContainer[n];
        _playerLabels = new SLabel[n];
    
        for (int cntr = 0; cntr < n; ++cntr) 
        {
            _playerContainers[cntr] = new SRadioContainer();
            temp = 5 * (cntr % 2) + 1;
            _playerLabels[cntr] = new SLabel(c,
                        _config.getStringProperty(PLAYERNAMES + "[" + cntr + "]", "Player " + (cntr+1)),
                        _playerCols[temp-1], _playerRows[cntr/2],
                        _playerCols[temp]-_playerCols[temp-1],
                        _playerRows[1]-_playerRows[0]);
/*            
            _playerLabels[cntr] = new SLabel(c,  "Player " + (cntr + 1), 
                            _playerCols[temp-1], _playerRows[cntr/2],
                            _playerCols[temp]-_playerCols[temp-1],
                            _playerRows[1]-_playerRows[0]);
*/
            pcolor = _config.getColorProperty(PLAYERCOLORS + "[" + cntr + "]", Color.white);
            _playerLabels[cntr].setFontColor(pcolor);
            for (int cntrRB = 0; cntrRB < 4; ++cntrRB) 
            {
                _radioButtons[cntr*4+cntrRB] = new SRadioButton(c, "", _playerCols[temp+cntrRB],
                                        _playerRows[cntr/2]);
                _radioButtons[cntr*4+cntrRB].setForeground(pcolor);
                _radioButtons[cntr*4+cntrRB].setBackground(Color.black);
                _playerContainers[cntr].addRadioButton(_radioButtons[cntr*4+cntrRB]);
            } 
        }

        // Explosion size buttons
        int tempRow = _config.getIntProperty(EXPLOSIONROW, 0);
        _explosionButtons = new SRadioButton[3];
        _explosionContainer = new SRadioContainer();
        String args[] = _config.parseString(_config.getStringProperty(EXPLOSIONCOLS, "0"), ',');
        for (int cntrExp = 0; cntrExp < 3; ++cntrExp)
        {
            try
            {
                temp = _config.parseInt(args[cntrExp], 200 * cntrExp);
            }
            catch (ArrayIndexOutOfBoundsException aioob)
            {
                temp = 200 * cntrExp;
            }
            _explosionButtons[cntrExp] = new SRadioButton(c,
                        _config.getStringProperty(EXPLOSIONLABELS + "[" + cntrExp + "]", "?"),
                        temp, tempRow);
            _explosionButtons[cntrExp].setForeground(Color.white);
            _explosionButtons[cntrExp].setBackground(Color.black);
            _explosionContainer.addRadioButton(_explosionButtons[cntrExp]);
        }
        _explosionContainer.setButton(2);
    }


// **********************************
// ***** METHODS - From Display *****
// **********************************

    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);

        int cntr;
        String args[] = _config.parseString(_config.getStringProperty(PLAYERROWS, "0"), ',');
        _playerRows = new int[args.length];
        for (cntr = 0; cntr < args.length; ++cntr)
            _playerRows[cntr] = _config.parseInt(args[cntr], 0);
    
        args = _config.parseString(_config.getStringProperty(PLAYERCOLS, "0"), ',');
        _playerCols = new int[args.length];
        for (cntr = 0; cntr < args.length; ++cntr)
            _playerCols[cntr] = _config.parseInt(args[cntr], 0);
    
        prepareRadioButtons();
    
        return true;
    }
    
    public void mouse_move(int x, int y, boolean b)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < _buttons.length; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_move(x, y, b);
        }
        for (cntr = 0; cntr < _playerContainers.length; ++cntr)
        {
            repaint |= _playerContainers[cntr].mouse_move(x, y, b);
        }
        repaint |= _explosionContainer.mouse_move(x, y, b);
        
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }
    
    public void mouse_down(int x, int y)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < _buttons.length; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_down(x, y);
        }
        for (cntr = 0; cntr < _playerContainers.length; ++cntr)
        {
            repaint |= _playerContainers[cntr].mouse_down(x, y);
        }
        repaint |= _explosionContainer.mouse_down(x, y);
        
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

    public void mouse_up(int x, int y)
    {
        boolean repaint = false;
        int cntr;

        for (cntr = 0; cntr < _buttons.length; ++cntr)
        {
            if (_buttons[cntr].mouse_up(x, y))
            {
                _displayHandler.displayEvent(this, _buttonActions[cntr]);
                repaint = true;
            }
        }
        for (cntr = 0; cntr < _playerContainers.length; ++cntr)
        {
            repaint |= _playerContainers[cntr].mouse_up(x, y);
        }
        repaint |= _explosionContainer.mouse_up(x, y);

        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }
    
    public synchronized void render(Graphics g, Component comp)
    {
        super.render(g, comp);

        for (int cntr = 0; cntr < _playerContainers.length; ++cntr) 
        {
            _playerContainers[cntr].render(g, comp);
            _playerLabels[cntr].render(g, comp);
        }
        _explosionContainer.render(g, comp);
    }


// ***************************************************************************
// ***** METHODS - Additional information getting (setting done by user) *****
// ***************************************************************************

    public int getPlayerType(int id)
    {
        return _playerContainers[id].getButton();
    }

    public int getExplosionRadius()
    {
        return _explosionContainer.getButton();
    }
}
