package com.blazinggames.retro.shared.search;

// Java imports
import java.awt.Point;

/**
* Tank class for use with the tank game.
 * ------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.00.011015
 */
public class Tank
{
    // *********************
    // ***** VARIABLES *****
    // *********************

    // coordinates
    protected double _x;
    protected double _y;
    protected int _direction; // (0-359)

    // firing related
    protected int _fireDirection; // (0-359)
    protected int _fireMinutes; // (0-59)
    protected int _fireSeconds; // (0-59)

    protected int _angle; // (0-90)
    protected int _power;

    // status related
    protected boolean _alive;


    // ************************
    // ***** CONSTRUCTORs *****
    // ************************

    /**
        * Default constructor.
     */
    public Tank()
    {
        this(0,0,0,0,0,0);
    }

    public Tank(double x, double y, int dir)
    {
        this(x, y, dir, 0, 0, 0);
    }

    public Tank(double x, double y, int dir, int fdir, int fang, int fpow)
    {
        _x = x;
        _y = y;
        _direction = (dir & Integer.MAX_VALUE) % 360;
        _fireDirection = (fdir & Integer.MAX_VALUE) % 360;
        if (fang <= 90)
            _angle = fang & Integer.MAX_VALUE;
        else
            _angle = (fang & Integer.MAX_VALUE) % 90;
        _power = fpow;
        _alive = true;
    }


    // *****************************************************
    // ***** METHODS - Information setting and getting *****
    // *****************************************************

    public void setLocation(double x, double y)
    {
        _x = x;
        _y = y;
    }

    public double getX()
    {
        return _x;
    }

    public double getY()
    {
        return _y;
    }

    public void setDirection(int n)
    {
        while (n < 0) {n += 360;}
        _direction = n % 360;
    }

    public int getDirection()
    {
        return _direction;
    }

    public void setFireDirection(int n)
    {
        _fireDirection = (n & Integer.MAX_VALUE) % 360;
        _fireMinutes = 0;
        _fireSeconds = 0;
    }

    public void setFireDirection(double d)
    {
        int n, m, s;
        double dtemp;
        n = (int)d;
        dtemp = d - (double)n;
        m = (int)(dtemp * 60);
        s = (int)((dtemp * 3600) % 60);
        //System.out.println("tank set fire dire set to " + n + "d" + m + "m" + s);
        setFireDirection(n, m, s);
    }

    public void setFireDirection(int n, int m, int s)
    {
        _fireDirection = (n & Integer.MAX_VALUE) % 360;
        _fireMinutes = m;
        _fireSeconds = s;
    }

    public int getFireDirection()
    {
        return _fireDirection;
    }

    public void setFireAngle(int n)
    {
        if (n <= 90)
            _angle = n & Integer.MAX_VALUE;
        else
            _angle = (n & Integer.MAX_VALUE) % 90;
    }

    public int getFireAngle()
    {
        return _angle;
    }

    public void setFirePower(int n)
    {
        _power = n;
    }

    public int getFirePower()
    {
        return _power;
    }

    public void setAlive(boolean b)
    {
        _alive = b;
    }

    public boolean isAlive()
    {
        return _alive;
    }


    // *****************************
    // ***** METHODS - Actions *****
    // *****************************

    public void turn(int deg)
    {
        setDirection(_direction + deg);
    }

    public void move(int units)
    {
        double rad = (double)_direction * Math.PI / 180.0;
        double xadj = Math.cos(rad) * units;
        double yadj = Math.sin(rad) * units;
        _x += xadj;
        _y += yadj;
    }

    public Projectile fire()
    {
        //        return new Projectile(_x, _y, _fireDirection, _angle, _power);
        double fireDir = (double)_fireDirection + ((double)_fireMinutes / 60) + ((double)_fireSeconds / 3600);
        //System.out.println("firing at dir " + fireDir);
        return new Projectile(_x, _y, fireDir, _angle, _power);
    }
}
