package com.blazinggames.urcl.ui.comp;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.sprite.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * Your basic text label, but taking advantage of transparent backgrounds (optional), and
 * different rendering styles (normal, shadow, outline).
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class SLabel
extends Comp
{
    // *********************
    // ***** CONSTANTS *****
    // *********************

    // formats (same as in TextFont, but here for convienience)
    public static final int FORMAT_PLAIN = 0;
    public static final int FORMAT_SHADOW = 1;
    public static final int FORMAT_OUTLINE = 2;

    // Alignments (also elsewhere, but here for convienience)
    public static final int ALIGN_LEFT = 0;
    public static final int ALIGN_CENTER = 1;
    public static final int ALIGN_RIGHT = 2;


    // *********************
    // ***** VARIABLES *****
    // *********************

    protected int _align;
    protected Color _background;
    protected boolean _solid;


    // ************************
    // ***** CONSTRUCTORs *****
    // ************************

    public SLabel(Component c, String s, Rectangle r)
    {
        super();
        _sprite = new TextSprite(c, s, r.x, r.y);
        setBounds(r);
        _align = 0;
        _background = Color.black;
        _solid = true;
    }

    public SLabel(Component c, String s, int x, int y, int w, int h)
    {
        super();
        _sprite = new TextSprite(c, s, x, y);
        setBounds(x, y, w, h);
        _align = 0;
        _background = Color.black;
        _solid = true;
    }


    // *******************************
    // ***** METHODS - From Comp *****
    // *******************************

    /**
        * Draws the component.
        *
        * @param g Graphics Context to draw into
        * @param c Component class that some awt commands need
        */
    public void render(Graphics g, Component c)
    {
        _changed = false;
        if (_isVisible == false)
            return;
        if (_solid)
        {
            g.setColor(_background);
            g.fillRect(_bounds.x, _bounds.y, _bounds.width, _bounds.height);
        }
        int y = _bounds.y + ((_bounds.height - _sprite.getHeight()) / 2) ;
        switch (_align)
        {
            case ALIGN_LEFT:
                _sprite.move(_bounds.x, y);
                break;

            case ALIGN_CENTER:
                _sprite.move(_bounds.x + (_bounds.width - _sprite.getWidth()) / 2, y);
                break;

            case ALIGN_RIGHT:
                _sprite.move(_bounds.x + _bounds.width - _sprite.getWidth(), y);
        }
        _sprite.setVisible(_isVisible);
        _sprite.render(g, c);
        _sprite.clean();
    }


    // *********************************************************
    // ***** METHODS - additional info getting and setting *****
    // *********************************************************

    /**
        */
    public void setText(Component c, String s)
    {
        ((TextSprite)_sprite).setText(c, s);
    }

    public void setFont(Component c, Font f)
    {
        ((TextSprite)_sprite).setFont(c, f);
    }

    public void setFormat(int n)
    {
        ((TextSprite)_sprite).setFormat(n);
    }

    public void setAlignment(int n)
    {
        _align = n;
    }

    public void setSolid(boolean b)
    {
        _solid = b;
        _sprite.setSolid(b);
    }

    public void setFontColor(Color c)
    {
        ((TextSprite)_sprite).setFontColor(c);
    }

    public void setHighlightColor(Color c)
    {
        ((TextSprite)_sprite).setHighlightColor(c);
    }

    public void setBackground(Color c)
    {
        _background = c;
        _sprite.setBackground(c);
    }

}
