package com.blazinggames.urcl.ui.comp;

// Java imports
import java.awt.*;
import java.util.Vector;

/**
 * This is what holds and controls a block of radio buttons.
 * The radio buttons are added (and assigned a number based on the
 * order they were added) and this class is used in place of the
 * separate radio button controls. It makes sure only one of the radio
 * buttons within the group is selected at a time, while handling
 * all the interface calls.
 *
 * @author Billy D. Spelchan
 * @version 1.10.031028
 */
public class SRadioContainer
extends Comp
{
// *********************
// ***** VARIABLES *****
// *********************

    protected Vector _radioList;
    protected int _selected;


// ************************
// ***** CONSTRUCTORS *****
// ************************

    public SRadioContainer()
    {
	super();
	_radioList = new Vector();
	_selected = 0;
    }


// *******************************
// ***** METHODS - From Comp *****
// *******************************

    /**
     * Called when mouse is moved. 
     *
     * @param x x coordinate of mouse pointer
     * @param y y coordinate of mouse pointer
     * @param b this is true if mouse button currently is down
     * @return true if mouse is over component.
    */
    public boolean mouse_move(int x, int y, boolean b)
    {
        if (_isActive) {
	    boolean result = false;
	    SRadioButton rb;
	    for (int cntr = 0; cntr < _radioList.size(); ++cntr) {
		rb = (SRadioButton)_radioList.elementAt(cntr);
		result |= rb.mouse_move(x, y, b);
	    }
	    return result;
	}
        else
            return false;
    }
    
    /**
     * Called when mouse has been pressed.
     * 
     * @param x x coordinate of where mouse was clicked
     * @param y y coordinate of where mouse was clicked
     * @return true if mouse went down within this component
    */
    public boolean mouse_down(int x, int y)
    {
        if (_isActive) {
	    boolean result = false;
	    SRadioButton rb;
	    for (int cntr = 0; cntr < _radioList.size(); ++cntr) {
		rb = (SRadioButton)_radioList.elementAt(cntr);
		result |= rb.mouse_down(x, y);
	    }
	    return result;
	}
        else
            return false;
    }
    
    /** 
     * Called when mouse button has been released.
     *
     * @param x x coordinate of where mouse was released
     * @param y y coordinate of where mouse was released
     * @return true if component clicked (pressed and released in component)
    */
    public boolean mouse_up(int x, int y)
    {
        if (_isActive) {
	    boolean result = false;
	    int newSelected = 0;
	    SRadioButton rb;
	    for (int cntr = 0; cntr < _radioList.size(); ++cntr) {
		rb = (SRadioButton)_radioList.elementAt(cntr);
		if (rb.mouse_up(x, y) == true) {
		    result = true;
		    newSelected = cntr;
		}
	    }
	    
	    if (result) {
		rb = (SRadioButton)_radioList.elementAt(_selected);
		rb.setState(0);
		rb = (SRadioButton)_radioList.elementAt(newSelected);
		rb.setState(1);
		_selected = newSelected;
		_changed = true;
	    }
	    return result;
	}
        else
            return false;
    }

    /**
     * Called at a regular interval. Useful for animation.
     *
     * @param ms number of milliseconds that passed since last tick.
     * @return true if state or image changed
    */
    public boolean tick(int ms)
    {
        if (_isActive) {
	    boolean result = false;
	    SRadioButton rb;
	    for (int cntr = 0; cntr < _radioList.size(); ++cntr) {
		rb = (SRadioButton)_radioList.elementAt(cntr);
		result |= rb.tick(ms);
	    }
	    return result;
	}
        else
            return false;
    }

    /**
     * Draws the component.
     *
     * @param g Graphics Context to draw into
     * @param c Component class that some awt commands need
    */
    public void render(Graphics g, Component c)
    {
        _changed = false;

        if (_isActive) 
	{
	    SRadioButton rb;
	    for (int cntr = 0; cntr < _radioList.size(); ++cntr) 
	    {
		rb = (SRadioButton)_radioList.elementAt(cntr);
		rb.render(g, c);
	    }
	}
    }


    // ************************************************
    // ***** METHODS - Radio button manipulations *****
    // ************************************************

    public void clear()
    {
	_selected = 0;
	_radioList.removeAllElements();
    }

    public void addRadioButton(SRadioButton rb)
    {
	_radioList.addElement(rb);
	if (_radioList.size() == 1)
	{
	    rb.setState(1);
	    _selected = 0;
	}
    }

    public void setButton(int n)
    {
	if (n >= _radioList.size())
	    return;
	SRadioButton rb = (SRadioButton)_radioList.elementAt(_selected);
	rb.setState(0);
	rb = (SRadioButton)_radioList.elementAt(n);
	rb.setState(1);
	_selected = n;
		_changed = true;
    }

    public int getButton()
    {
	return _selected;
    }
}
